﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace XamScheduleTest
{
	public class ScheduleData
	{
		private ObservableCollection<ResourceItem> _resources;
		private ObservableCollection<AppointmentItem> _appointments;

		public ScheduleData()
		{
			_resources = new ObservableCollection<ResourceItem>();
			_appointments = new ObservableCollection<AppointmentItem>();

			_resources.Add(new ResourceItem { Id = "0", Name = "John Doe" });
			_resources.Add(new ResourceItem { Id = "1", Name = "Jane Doe" });

			_appointments.Add(new AppointmentItem { Id = "0", OwningResourceId = "0", Start = DateTime.Today.ToUniversalTime().AddHours(9), End = DateTime.Today.ToUniversalTime().AddHours(9.5), Subject="First", Comments = "Originally started at 9am" });
			_appointments.Add(new AppointmentItem { Id = "1", OwningResourceId = "0", Start = DateTime.Today.ToUniversalTime().AddHours(10), End = DateTime.Today.ToUniversalTime().AddHours(10.5), Subject = "Second", Comments = "Originally started at 10am" });
			_appointments.Add(new AppointmentItem { Id = "2", OwningResourceId = "0", Start = DateTime.Today.ToUniversalTime().AddHours(11), End = DateTime.Today.ToUniversalTime().AddHours(11), Subject = "Third", Comments = "Originally 0 minute appt at 11am" });
		}

		public ObservableCollection<ResourceItem> Resources
		{
			get { return _resources; }
		}

		public ObservableCollection<AppointmentItem> Appointments
		{
			get { return _appointments; }
		}
	}

	public class DataItemBase : INotifyPropertyChanged
	{
		#region SetField
		protected bool SetField<T>(ref T member, T newValue, string propertyName)
		{
			if (!EqualityComparer<T>.Default.Equals(member, newValue))
			{
				member = newValue;
				this.OnPropertyChanged(propertyName);
				return true;
			}

			return false;
		} 
		#endregion //SetField

		#region OnPropertyChanged
		protected void OnPropertyChanged(string propertyName)
		{
			var handler = this.PropertyChanged;

			if (null != handler)
				handler(this, new PropertyChangedEventArgs(propertyName));
		} 
		#endregion //OnPropertyChanged

		#region Events

		public event PropertyChangedEventHandler PropertyChanged; 

		#endregion //Events
	}

	public class ResourceItem : DataItemBase
	{
		#region Id
		private string _id;
		public string Id
		{
			get { return _id; }
			set { this.SetField(ref _id, value, "Id"); }
		} 
		#endregion //Id

		#region Name
		private string _name;
		public string Name
		{
			get { return _name; }
			set { this.SetField(ref _name, value, "Name"); }
		}
		#endregion //Name
	}

	public class AppointmentItem : DataItemBase
	{
		#region Id
		private string _id;
		public string Id
		{
			get { return _id; }
			set { this.SetField(ref _id, value, "Id"); }
		}
		#endregion //Id

		#region OwningResourceId
		private string _owningResourceId;
		public string OwningResourceId
		{
			get { return _owningResourceId; }
			set { this.SetField(ref _owningResourceId, value, "OwningResourceId"); }
		}
		#endregion //OwningResourceId

		#region Start
		private DateTime _start;
		public DateTime Start
		{
			get { return _start; }
			set { this.SetField(ref _start, value, "Start"); }
		} 
		#endregion //Start

		#region End
		private DateTime _end;
		public DateTime End
		{
			get { return _end; }
			set { this.SetField(ref _end, value, "End"); }
		}
		#endregion //End

		#region Subject
		private string _subject;
		public string Subject
		{
			get { return _subject; }
			set { this.SetField(ref _subject, value, "Subject"); }
		}
		#endregion //Subject

		#region Comments
		private string _comments;
		public string Comments
		{
			get { return _comments; }
			set { this.SetField(ref _comments, value, "Comments"); }
		}
		#endregion //Comments
	}
}
